@@ -1,7 +1,9 @@
+use core::marker::PhantomData;
+
 use alloc::vec::Vec;
 
 use crate::{
-    api::{ErrorApi, ManagedTypeApi},
+    api::{ErrorApi, ErrorApiImpl, ManagedTypeApi},
     err_msg,
     types::{BoxedBytes, ManagedBytesTopDecodeInput},
     DynArgInput,
@@ -10,22 +12,22 @@ use crate::{
 /// Consumes a vector of `BoxedBytes` and deserializes from the vector one by one.
 pub struct BytesArgLoader<A>
 where
-    A: ManagedTypeApi + ErrorApi,
+    A: ManagedTypeApi,
 {
     bytes_vec: Vec<BoxedBytes>,
     next_index: usize,
-    api: A,
+    _phantom: PhantomData<A>,
 }
 
 impl<A> BytesArgLoader<A>
 where
-    A: ManagedTypeApi + ErrorApi,
+    A: ManagedTypeApi,
 {
-    pub fn new(api: A, bytes_vec: Vec<BoxedBytes>) -> Self {
+    pub fn new(bytes_vec: Vec<BoxedBytes>) -> Self {
         BytesArgLoader {
             bytes_vec,
             next_index: 0,
-            api,
+            _phantom: PhantomData,
         }
     }
 }
@@ -36,12 +38,7 @@ where
 {
     type ItemInput = ManagedBytesTopDecodeInput<A>;
 
-    type ErrorApi = A;
-
-    #[inline]
-    fn dyn_arg_vm_api(&self) -> Self::ErrorApi {
-        self.api.clone()
-    }
+    type ManagedTypeErrorApi = A;
 
     #[inline]
     fn has_next(&self) -> bool {
@@ -50,8 +47,7 @@ where
 
     fn next_arg_input(&mut self) -> ManagedBytesTopDecodeInput<A> {
         if !self.has_next() {
-            self.dyn_arg_vm_api()
-                .signal_error(err_msg::ARG_WRONG_NUMBER);
+            A::error_api_impl().signal_error(err_msg::ARG_WRONG_NUMBER);
         }
 
         // consume from the vector, get owned bytes
@@ -60,6 +56,6 @@ where
         let boxed_bytes =
             core::mem::replace(&mut self.bytes_vec[self.next_index], BoxedBytes::empty());
         self.next_index += 1;
-        ManagedBytesTopDecodeInput::new(self.api.clone(), boxed_bytes)
+        ManagedBytesTopDecodeInput::new(boxed_bytes)
     }
 }
