@@ -1,14 +1,15 @@
+use core::marker::PhantomData;
+
 use super::{StorageClearable, StorageMapper};
 use crate::{
     abi::{TypeAbi, TypeDescriptionContainer, TypeName},
-    api::{EndpointFinishApi, ErrorApi, ManagedTypeApi, StorageReadApi, StorageWriteApi},
+    api::{EndpointFinishApi, ManagedTypeApi, StorageMapperApi},
     finish_all,
     io::EndpointResult,
     storage::{storage_get, storage_set, StorageKey},
-    types::MultiResultVec,
+    types::{ManagedType, MultiResultVec},
 };
 use alloc::vec::Vec;
-use core::marker::PhantomData;
 use elrond_codec::{
     elrond_codec_derive::{TopDecode, TopDecodeOrDefault, TopEncode, TopEncodeOrDefault},
     DecodeDefault, EncodeDefault, TopDecode, TopEncode,
@@ -63,31 +64,31 @@ impl QueueMapperInfo {
 /// in constant time.
 pub struct QueueMapper<SA, T>
 where
-    SA: StorageReadApi + StorageWriteApi + ManagedTypeApi + ErrorApi + Clone + 'static,
+    SA: StorageMapperApi,
     T: TopEncode + TopDecode + 'static,
 {
-    api: SA,
+    _phantom_api: PhantomData<SA>,
     base_key: StorageKey<SA>,
-    _phantom: core::marker::PhantomData<T>,
+    _phantom_item: PhantomData<T>,
 }
 
 impl<SA, T> StorageMapper<SA> for QueueMapper<SA, T>
 where
-    SA: StorageReadApi + StorageWriteApi + ManagedTypeApi + ErrorApi + Clone + 'static,
+    SA: StorageMapperApi,
     T: TopEncode + TopDecode,
 {
-    fn new(api: SA, base_key: StorageKey<SA>) -> Self {
+    fn new(base_key: StorageKey<SA>) -> Self {
         QueueMapper {
-            api,
+            _phantom_api: PhantomData,
             base_key,
-            _phantom: PhantomData,
+            _phantom_item: PhantomData,
         }
     }
 }
 
 impl<SA, T> StorageClearable for QueueMapper<SA, T>
 where
-    SA: StorageReadApi + StorageWriteApi + ManagedTypeApi + ErrorApi + Clone + 'static,
+    SA: StorageMapperApi,
     T: TopEncode + TopDecode,
 {
     fn clear(&mut self) {
@@ -105,7 +106,7 @@ where
 
 impl<SA, T> QueueMapper<SA, T>
 where
-    SA: StorageReadApi + StorageWriteApi + ManagedTypeApi + ErrorApi + Clone + 'static,
+    SA: StorageMapperApi,
     T: TopEncode + TopDecode,
 {
     fn build_node_id_named_key(&self, name: &[u8], node_id: u32) -> StorageKey<SA> {
@@ -122,44 +123,40 @@ where
     }
 
     fn get_info(&self) -> QueueMapperInfo {
-        storage_get(self.api.clone(), &self.build_name_key(INFO_IDENTIFIER))
+        storage_get(self.build_name_key(INFO_IDENTIFIER).as_ref())
     }
 
     fn set_info(&mut self, value: QueueMapperInfo) {
-        storage_set(
-            self.api.clone(),
-            &self.build_name_key(INFO_IDENTIFIER),
-            &value,
-        );
+        storage_set(self.build_name_key(INFO_IDENTIFIER).as_ref(), &value);
     }
 
     fn get_node(&self, node_id: u32) -> Node {
         storage_get(
-            self.api.clone(),
-            &self.build_node_id_named_key(NODE_IDENTIFIER, node_id),
+            self.build_node_id_named_key(NODE_IDENTIFIER, node_id)
+                .as_ref(),
         )
     }
 
     fn set_node(&mut self, node_id: u32, item: Node) {
         storage_set(
-            self.api.clone(),
-            &self.build_node_id_named_key(NODE_IDENTIFIER, node_id),
+            self.build_node_id_named_key(NODE_IDENTIFIER, node_id)
+                .as_ref(),
             &item,
         );
     }
 
     fn clear_node(&mut self, node_id: u32) {
         storage_set(
-            self.api.clone(),
-            &self.build_node_id_named_key(NODE_IDENTIFIER, node_id),
+            self.build_node_id_named_key(NODE_IDENTIFIER, node_id)
+                .as_ref(),
             &(),
         );
     }
 
     fn get_value(&self, node_id: u32) -> T {
         storage_get(
-            self.api.clone(),
-            &self.build_node_id_named_key(VALUE_IDENTIFIER, node_id),
+            self.build_node_id_named_key(VALUE_IDENTIFIER, node_id)
+                .as_ref(),
         )
     }
 
@@ -172,16 +169,16 @@ where
 
     fn set_value(&mut self, node_id: u32, value: &T) {
         storage_set(
-            self.api.clone(),
-            &self.build_node_id_named_key(VALUE_IDENTIFIER, node_id),
+            self.build_node_id_named_key(VALUE_IDENTIFIER, node_id)
+                .as_ref(),
             value,
         )
     }
 
     fn clear_value(&mut self, node_id: u32) {
         storage_set(
-            self.api.clone(),
-            &self.build_node_id_named_key(VALUE_IDENTIFIER, node_id),
+            self.build_node_id_named_key(VALUE_IDENTIFIER, node_id)
+                .as_ref(),
             &(),
         )
     }
@@ -414,7 +411,7 @@ where
 /// documentation for more.
 pub struct Iter<'a, SA, T>
 where
-    SA: StorageReadApi + StorageWriteApi + ManagedTypeApi + ErrorApi + Clone + 'static,
+    SA: StorageMapperApi,
     T: TopEncode + TopDecode + 'static,
 {
     node_id: u32,
@@ -423,7 +420,7 @@ where
 
 impl<'a, SA, T> Iter<'a, SA, T>
 where
-    SA: StorageReadApi + StorageWriteApi + ManagedTypeApi + ErrorApi + Clone + 'static,
+    SA: StorageMapperApi,
     T: TopEncode + TopDecode + 'static,
 {
     fn new(queue: &'a QueueMapper<SA, T>) -> Iter<'a, SA, T> {
@@ -436,7 +433,7 @@ where
 
 impl<'a, SA, T> Iterator for Iter<'a, SA, T>
 where
-    SA: StorageReadApi + StorageWriteApi + ManagedTypeApi + ErrorApi + Clone + 'static,
+    SA: StorageMapperApi,
     T: TopEncode + TopDecode + 'static,
 {
     type Item = T;
@@ -455,23 +452,23 @@ where
 /// Behaves like a MultiResultVec when an endpoint result.
 impl<SA, T> EndpointResult for QueueMapper<SA, T>
 where
-    SA: StorageReadApi + StorageWriteApi + ManagedTypeApi + ErrorApi + Clone + 'static,
+    SA: StorageMapperApi,
     T: TopEncode + TopDecode + EndpointResult,
 {
     type DecodeAs = MultiResultVec<T::DecodeAs>;
 
-    fn finish<FA>(&self, api: FA)
+    fn finish<FA>(&self)
     where
-        FA: ManagedTypeApi + EndpointFinishApi + Clone + 'static,
+        FA: ManagedTypeApi + EndpointFinishApi,
     {
-        finish_all(api, self.iter());
+        finish_all::<FA, _, _>(self.iter());
     }
 }
 
 /// Behaves like a MultiResultVec when an endpoint result.
 impl<SA, T> TypeAbi for QueueMapper<SA, T>
 where
-    SA: StorageReadApi + StorageWriteApi + ManagedTypeApi + ErrorApi + Clone + 'static,
+    SA: StorageMapperApi,
     T: TopEncode + TopDecode + TypeAbi,
 {
     fn type_name() -> TypeName {
